/*  Go benchmarks to compare against Python and Node.js
 *
 *  @author     Dave Lasley, dave@dlasley.net
 *  @website    https://blog.dlasley.net
 */

package main

import (
    "bytes"
    "flag"
    "fmt"
)

/*  Test str concatenation, generic method  */
func strConcat() string{
    var str string
    
    for i := 0; i < 100000; i++{
        str += "A"
    }
    
    return str
}

/*  Language specific recommended way of str concat */
func strConcatRecommended() string{
    var buffer bytes.Buffer

    for i := 0; i < 100000; i++ {
        buffer.WriteString("A")
    }

    return buffer.String()
}

/*  Int math+reassignment   */
func intBench() int{
    var test int
    
    for i := 0; i < 100000; i++{
        test += i
    }
    
    return test
}

/*  Array assignments   */
func arrBench() [100][100]int{
    var test [100][100]int
    
    //  Loop 100 assignments of all elements
    for i := 0; i < 1000; i++{
        for j := 0; j < 100; j++{
            for k := 0; k < 100; k++{
                test[j][k] = i+j+k
            }
        }
    }
    
    return test
}

/*  Slice assignments   */
func sliceBench() []int{
    
    test := make([]int, 100)
    
    //  Loop 1000 iters of 100 assignments of all elements
    for i := 0; i < 100000; i++{
        
        for j := 0; j < 100; j++{
            test[j] = j+i
        }
        
    }
    
    return test
}

/*  Fib */
func fib(n int) int{
    if(n<2){
        return n
    }
    return fib(n-2) + fib(n-1)
}

func main(){
    str := flag.Bool("str", false, "str concat test")
    buffer := flag.Bool("strrec", false, "buffer write test")
    int_ := flag.Bool("int", false, "int test")
    arr := flag.Bool("arr", false, "array test")
    slice := flag.Bool("slice", false, "slice test")
    fib_ := flag.Bool("fib", false, "fib test")
    
    flag.Parse()
    
    if *str{
        fmt.Println(strConcat())
    }
    
    if *buffer{
        fmt.Println(strConcatRecommended())
    }
    
    if *int_{
        fmt.Println(intBench())
    }
    
    if *arr{
        fmt.Println(arrBench())
    }
    
    if *slice{
        fmt.Println(sliceBench())
    }
    
    if *fib_{
        fmt.Println(fib(40))
    }
}